/*
 GBxCart RW - Console Interface Flasher
 Version: 1.25
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 26/08/2017
 Last Modified: 2/12/2019
 License: GPL
 
 This program allows you to write ROMs to Flash Carts that are supported.
 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <time.h>

#ifdef _WIN32
#include <Windows.h>
#else
#include <unistd.h>
#endif

#include "setup.h" // See defines, variables, constants, functions here

int main(int argc, char **argv) {
	
	printf("GBxCart RW Flasher v1.25 by insideGadgets\n");
	printf("#########################################\n");
	
	// Check arguments
	if (argc >= 2) {
		read_config();
		
		// Open COM port
		if (com_test_port() == 0) {
			printf("Device not connected and couldn't be auto detected\n");
			read_one_letter();
			return 1;
		}
		printf("Connected on COM port: %i\n", cport_nr+1);
		
		// Break out of any existing functions on ATmega
		set_mode('0');
		
		// Get cartridge mode - Gameboy or Gameboy Advance
		cartridgeMode = request_value(CART_MODE);
		
		// Get firmware version
		gbxcartFirmwareVersion = request_value(READ_FIRMWARE_VERSION);
		
		// Get PCB version
		gbxcartPcbVersion = request_value(READ_PCB_VERSION);
		
		if (gbxcartPcbVersion == PCB_1_0) {
			printf("\nPCB v1.0 is not supported for this function.");
			read_one_letter();
			return 1;
		}
		
		if (gbxcartFirmwareVersion <= 8) {
			printf("Firmware R9 or higher is required for this functionality.\n");
			read_one_letter();
			return 1;
		}
		
		
		// Separate the file name
		char filenameOnly[100];
		char filename[254];
		strncpy(filename, argv[1], 253);
		
		char *token = strtok(filename, "\\");
		while (token != NULL) {
			strncpy(filenameOnly, token, 99);
			token = strtok(NULL, "\\");
		}
		
		printf("\n--- Write ROM to Flash Cart ---\n");
		
		
		uint32_t readBytes = 0;
		uint16_t sector = 0;
		long fileSize = 0;
		
		// Grab file size
		FILE *romFile = fopen(argv[1], "rb");
		if (romFile != NULL) {
			fseek(romFile, 0, SEEK_END);
			fileSize = ftell(romFile);
			fseek(romFile, 0, SEEK_SET);
		}
		else {
			printf("\n%s \nFile not found\n", argv[1]);
			read_one_letter();
			return 1;
		}
		
		currAddr = 0x0000;
		endAddr = 0x7FFF;
		
		// PCB v1.3 - Set 5V
		if (gbxcartPcbVersion == PCB_1_3) {
			set_mode(VOLTAGE_5V);
			delay_ms(500);
		}
		
		// Check file size
		if (fileSize > (endAddr+1)) {
			fclose(romFile);
			printf("\n%s \nFile size is larger than the available Flash cart space of 32K\n", argv[1]);
			read_one_letter();
			return 1;
		}
		
		// Flash Setup
		set_mode(GB_CART_MODE); // Gameboy mode
		gb_flash_pin_setup(WE_AS_AUDIO_PIN); // Audio pin
		gb_flash_program_setup(GB_FLASH_PROGRAM_5555);// Flash program byte method
		gb_check_change_flash_id(GB_FLASH_PROGRAM_5555);
		
		
		printf("\nWriting to ROM (Flash cart) from %s\n", filenameOnly);
		printf("[             25%%             50%%             75%%            100%%]\n[");
		
		
		// Write ROM
		set_number(currAddr, SET_START_ADDRESS);
		while (currAddr <= endAddr) {
			com_write_bytes_from_file(GB_FLASH_WRITE_64BYTE, romFile, 64);
			com_wait_for_ack();
			currAddr += 64;
			readBytes += 64;
			
			print_progress_percent(readBytes, (endAddr+1) / 64);
		}
		
		printf("]");
		fclose(romFile);
		
	}
	
	printf("\n");
	
	return 0;
}